@@ -185,9 +185,14 @@ msgstr ""
         """
         Test with utf8 data (no file).
         """
-        f = open('tests/test_utf8.po','r')
+        if polib.PY3:
+            f = open('tests/test_utf8.po', 'rb')
+            data = str(f.read(), 'utf-8')
+        else:
+            f = open('tests/test_utf8.po', 'r')
+            data = f.read()
         try:
-            self.assertEqual(polib.detect_encoding(f.read()), 'UTF-8')
+            self.assertEqual(polib.detect_encoding(data), 'UTF-8')
         finally:
             f.close()    
 
@@ -264,9 +269,21 @@ class TestBaseFile(unittest.TestCase):
     def test_append2(self):
         def add_duplicate():
             pofile = polib.pofile('tests/test_pofile_helpers.po', check_for_duplicates=True)
-            pofile.append(polib.POEntry(msgid="and", msgstr="y"))
+            pofile.append(polib.POEntry(msgid="and"))
         self.assertRaises(ValueError, add_duplicate)
 
+    def test_append3(self):
+        def add_duplicate():
+            pofile = polib.pofile('tests/test_pofile_helpers.po', check_for_duplicates=True)
+            pofile.append(polib.POEntry(msgid="and", msgctxt="some context"))
+        self.assertRaises(ValueError, add_duplicate)
+
+    def test_append4(self):
+        pofile = polib.pofile('tests/test_pofile_helpers.po', check_for_duplicates=True)
+        entry = polib.POEntry(msgid="and", msgctxt="some different context")
+        pofile.append(entry)
+        self.assertTrue(entry in pofile)
+
     def test_insert1(self):
         pofile = polib.pofile('tests/test_pofile_helpers.po')
         entry = polib.POEntry(msgid="Foo", msgstr="Bar", msgctxt="Some context")
@@ -445,6 +462,13 @@ class TestPoFile(unittest.TestCase):
         """
         Test for the POFile.save_as_mofile() method.
         """
+        import distutils.spawn
+        msgfmt = distutils.spawn.find_executable('msgfmt')
+        if msgfmt is None:
+            try:
+                return unittest.skip('msgfmt is not installed')
+            except AttributeError:
+                return
         reffiles = ['tests/test_utf8.po', 'tests/test_iso-8859-15.po']
         encodings = ['utf-8', 'iso-8859-15']
         for reffile, encoding in zip(reffiles, encodings):
@@ -454,7 +478,7 @@ class TestPoFile(unittest.TestCase):
             os.close(fd)
             po = polib.pofile(reffile, autodetect_encoding=False, encoding=encoding)
             po.save_as_mofile(tmpfile1)
-            subprocess.call(['msgfmt', '--no-hash', '-o', tmpfile2, reffile])
+            subprocess.call([msgfmt, '--no-hash', '-o', tmpfile2, reffile])
             try:
                 f = open(tmpfile1, 'rb')
                 s1 = f.read()
@@ -476,13 +500,13 @@ class TestPoFile(unittest.TestCase):
 
     def test_percent_translated(self):
         po = polib.pofile('tests/test_pofile_helpers.po')
-        self.assertEqual(po.percent_translated(), 50)
+        self.assertEqual(po.percent_translated(), 53)
         po = polib.POFile()
         self.assertEqual(po.percent_translated(), 100)
 
     def test_translated_entries(self):
         po = polib.pofile('tests/test_pofile_helpers.po')
-        self.assertEqual(len(po.translated_entries()), 6)
+        self.assertEqual(len(po.translated_entries()), 7)
 
     def test_untranslated_entries(self):
         po = polib.pofile('tests/test_pofile_helpers.po')
@@ -501,6 +525,11 @@ class TestPoFile(unittest.TestCase):
         self.assertNotEqual(po.metadata, {})
         self.assertEqual(po.metadata['Content-Type'], 'text/plain; charset=UTF-8')
 
+    def test_comment_starting_with_two_hashes(self):
+        po = polib.pofile('tests/test_utf8.po')
+        e = po.find("Some comment starting with two '#'", by='tcomment')
+        self.assertTrue(isinstance(e, polib.POEntry))
+
 class TestMoFile(unittest.TestCase):
     """
     Tests for MoFile class.
@@ -525,10 +554,16 @@ class TestMoFile(unittest.TestCase):
         mo = polib.mofile('tests/test_utf8.mo', wrapwidth=78)
         mo.save_as_pofile(tmpfile)
         try:
-            f = open(tmpfile)
+            if polib.PY3:
+                f = open(tmpfile, encoding='utf-8')
+            else:
+                f = open(tmpfile)
             s1 = f.read()
             f.close()
-            f = open('tests/test_save_as_pofile.po')
+            if polib.PY3:
+                f = open('tests/test_save_as_pofile.po', encoding='utf-8')
+            else:
+                f = open('tests/test_save_as_pofile.po')
             s2 = f.read()
             f.close()
             self.assertEqual(s1, s2)
